/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#if !defined(_LOG4CXX_TESTCHAR_H)
#define _LOG4CXX_TESTCHAR_H

//
//   This macro and typedef lets many tests
//      check either the wchar_t or char based methods.
//
#if LOG4CXX_WCHAR_T_API && defined(LOG4CXX_TEST_WCHAR)
	#define LOG4CXX_TEST_STR(str) L ## str
	typedef wchar_t testchar;
#else
	#define LOG4CXX_TEST_STR(str) str
	typedef char testchar;
#endif

#endif
